@@ -14,12 +14,11 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
    */ 
 
 #ifndef __NSAutoreleasePool_h_GNUSTEP_BASE_INCLUDE
@@ -53,7 +52,7 @@ typedef struct autorelease_thread_vars
   /* The current, default NSAutoreleasePool for the calling thread;
      the one that will hold objects that are arguments to
      [NSAutoreleasePool +addObject:]. */
-  NSAutoreleasePool *current_pool;
+  __unsafe_unretained NSAutoreleasePool *current_pool;
 
   /* The total number of objects autoreleased since the thread was
      started, or since -resetTotalAutoreleasedObjects was called
@@ -62,7 +61,7 @@ typedef struct autorelease_thread_vars
 
   /* A cache of NSAutoreleasePool's already alloc'ed.  Caching old pools
      instead of deallocating and re-allocating them will save time. */
-  id *pool_cache;
+  __unsafe_unretained id *pool_cache;
   int pool_cache_size;
   int pool_cache_count;
 } thread_vars_struct;
@@ -92,7 +91,7 @@ typedef struct autorelease_array_list
   struct autorelease_array_list *next;
   unsigned size;
   unsigned count;
-  id objects[0];
+  __unsafe_unretained id objects[0];
 } array_list_struct;
 
 
@@ -178,8 +177,10 @@ typedef struct autorelease_array_list
  *   </desc>
  * </deflist>
  */
-@interface NSAutoreleasePool : NSObject 
+GS_EXPORT_CLASS NS_AUTOMATED_REFCOUNT_UNAVAILABLE
+@interface NSAutoreleasePool : NSObject
 {
+#if	GS_EXPOSE(NSAutoreleasePool) && !__has_feature(objc_arc)
   /* For re-setting the current pool when we are dealloc'ed. */
   NSAutoreleasePool *_parent;
   /* This pointer to our child pool is  necessary for co-existing
@@ -192,6 +193,16 @@ typedef struct autorelease_array_list
   unsigned _released_count;
   /* The method to add an object to this pool */
   void 	(*_addImp)(id, SEL, id);
+#endif
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
+#endif
 }
 
 /**
@@ -219,10 +230,12 @@ typedef struct autorelease_array_list
  */
 - (id) autorelease;
 
-#if OS_API_VERSION(100400, GS_API_LATEST)
-/** Acts like the -release method.<br/>
- * Intended to trigger a garbage collection run if in a garbage collected
- * environment.
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
+/**
+ * Intended to trigger a garbage collection run (if needed) when called in
+ * a garbage collected environment.<br />
+ * In a non-garbage collected environment, this method implements the
+ * undocumented MacOS-X behavior, and releases the receiver.
  */
 - (void) drain;
 #endif
@@ -293,7 +306,7 @@ typedef struct autorelease_array_list
  *   Default value is maxint.
  * </p>
  */
-+ (void) setPoolCountThreshhold: (unsigned)c;
++ (void) setPoolCountThreshold: (unsigned)c;
 
 /**
  * Return the number of objects in this pool.
