@@ -1,26 +1,25 @@
-/* Interface for NSData for GNUStep
+/**Interface for NSData for GNUStep
    Copyright (C) 1995 Free Software Foundation, Inc.
 
    Written by:  Andrew Kachites McCallum <mccallum@gnu.ai.mit.edu>
    Date: 1995
-   
+
    This file is part of the GNUstep Base Library.
-   
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
+   Lesser General Public License for more details.
+
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
-   */ 
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
+   */
 
 #ifndef __NSData_h_GNUSTEP_BASE_INCLUDE
 #define __NSData_h_GNUSTEP_BASE_INCLUDE
@@ -29,6 +28,7 @@
 #import	<Foundation/NSObject.h>
 #import	<Foundation/NSRange.h>
 #import	<Foundation/NSSerialization.h>
+#import <GNUstepBase/GSBlocks.h>
 
 #if	defined(__cplusplus)
 extern "C" {
@@ -39,17 +39,48 @@ extern "C" {
 @class	NSURL;
 #endif
 
-#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST)
+enum {
+  NSDataSearchBackwards = (1UL << 0),
+  NSDataSearchAnchored = (1UL << 1),
+};
+typedef NSUInteger NSDataSearchOptions;
+#endif
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_9,GS_API_LATEST)
+enum {
+  NSDataBase64DecodingIgnoreUnknownCharacters = (1UL << 0)
+};
+typedef NSUInteger NSDataBase64DecodingOptions;
+
+enum {
+  NSDataBase64Encoding64CharacterLineLength = (1UL << 0),
+  NSDataBase64Encoding76CharacterLineLength = (1UL << 1),
+  NSDataBase64EncodingEndLineWithCarriageReturn = (1UL << 4),
+  NSDataBase64EncodingEndLineWithLineFeed = (1UL << 5),
+};
+typedef NSUInteger NSDataBase64EncodingOptions;
+#endif
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST)
 enum {
   NSMappedRead = 1,
   NSUncachedRead = 2
 };
 
 enum {
-  NSAtomicWrite = 1
+  NSDataWritingAtomic = 1
 };
+/* The original name for this was NSAtomicWrite ... need for backward comapat
+ */
+#define NSAtomicWrite   NSDataWritingAtomic
 #endif
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_9,GS_API_LATEST)
+DEFINE_BLOCK_TYPE(GSDataDeallocatorBlock, void, void*, NSUInteger);
+#endif
+
+GS_EXPORT_CLASS
 @interface NSData : NSObject <NSCoding, NSCopying, NSMutableCopying>
 
 // Allocating and Initializing a Data Object
@@ -70,6 +101,22 @@ enum {
 + (id) dataWithContentsOfURL: (NSURL*)url;
 #endif
 + (id) dataWithData: (NSData*)data;
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_9,GS_API_LATEST)
+- (id) initWithBase64EncodedData: (NSData*)base64Data
+                         options: (NSDataBase64DecodingOptions)options;
+- (id) initWithBase64EncodedString: (NSString*)base64String
+                           options: (NSDataBase64DecodingOptions)options;
+/**
+ * <override-subclass/>
+ * Initialize the receiver to hold memory pointed to by bytes without copying.
+ * When the receiver is deallocated, the memory will be freed using the user
+ * supplied deallocBlock. Note that passing a block that (either directly or
+ * indirectly) holds a strong reference the receiver will cause a retain cycle. 
+ */
+- (instancetype) initWithBytesNoCopy: (void*)bytes
+                              length: (NSUInteger)length
+                         deallocator: (GSDataDeallocatorBlock)deallocBlock;
+#endif
 - (id) initWithBytes: (const void*)aBuffer
 	      length: (NSUInteger)bufferSize;
 - (id) initWithBytesNoCopy: (void*)aBuffer
@@ -86,7 +133,7 @@ enum {
 #endif
 - (id) initWithData: (NSData*)data;
 
-// Accessing Data 
+// Accessing Data
 
 - (const void*) bytes;
 - (NSString*) description;
@@ -97,6 +144,18 @@ enum {
 	    range: (NSRange)aRange;
 - (NSData*) subdataWithRange: (NSRange)aRange;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST)
+- (NSRange) rangeOfData: (NSData *)dataToFind
+                options: (NSDataSearchOptions)mask
+                  range: (NSRange)searchRange;
+#endif
+
+// base64
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_9,GS_API_LATEST)
+- (NSData *) base64EncodedDataWithOptions: (NSDataBase64EncodingOptions)options;
+- (NSString *) base64EncodedStringWithOptions: (NSDataBase64EncodingOptions)options;
+#endif
+
 // Querying a Data Object
 
 - (BOOL) isEqualToData: (NSData*)other;
@@ -141,10 +200,10 @@ enum {
 		   count: (unsigned int)numInts
 		 atIndex: (unsigned int)index;
 
-#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST)
 /**
  * <p>Writes a copy of the data encapsulated by the receiver to a file
- * at path.  If the NSAtomicWrite option is set, this writes to a
+ * at path.  If the NSDataWritingAtomic option is set, this writes to a
  * temporary file and then renames that to the file at path, thus
  * ensuring that path exists and does not contain partially written
  * data at any point.
@@ -227,6 +286,7 @@ enum {
 #define	_GSC_ULNG_LNG	0x0a
 #define	_GSC_FLT	0x0b
 #define	_GSC_DBL	0x0c
+#define	_GSC_BOOL       0x0d
 
 #define	_GSC_ID		0x10
 #define	_GSC_CLASS	0x11
@@ -252,6 +312,7 @@ enum {
 @end
 #endif
 
+GS_EXPORT_CLASS
 @interface NSMutableData :  NSData
 
 + (id) dataWithCapacity: (NSUInteger)numBytes;
