@@ -14,12 +14,11 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
    */ 
 #ifndef __NSHost_h_GNUSTEP_BASE_INCLUDE
 #define __NSHost_h_GNUSTEP_BASE_INCLUDE
@@ -37,6 +36,7 @@ extern "C" {
  *  Instances of this class encapsulate host information.  Constructors based
  *  on host name or numeric address are provided.
  */
+GS_EXPORT_CLASS
 @interface NSHost : NSObject
 {
 #if	GS_EXPOSE(NSHost)
@@ -99,6 +99,14 @@ extern "C" {
  */
 - (BOOL) isEqualToHost: (NSHost*) aHost;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST) 
+/**
+ * If the receiver is the currentHost, returns the default name of the
+ * computer, otherwise returns nil.
+ */
+- (NSString*) localizedName;
+#endif
+
 /**
  * Return host name.  Chosen arbitrarily if a host has more than one.
  */
