@@ -14,12 +14,11 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    AutogsdocSource: NSIndexSet.m
 
@@ -44,6 +43,7 @@ extern "C" {
  * range 0 to NSNotFound-1.<br />
  * Each integer can appear in a collection only once.
  */
+GS_EXPORT_CLASS
 @interface	NSIndexSet : NSObject <NSCopying, NSMutableCopying, NSCoding>
 {
 #if	GS_EXPOSE(NSIndexSet)
@@ -90,28 +90,41 @@ extern "C" {
  */
 - (NSUInteger) count;
 
-#if OS_API_VERSION(100500,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST)
 /** Not implemented
  * Returns the number of indexes set within the specified range.
  */
 - (NSUInteger) countOfIndexesInRange: (NSRange)range;
 #endif
 
-#if OS_API_VERSION(100600,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST)
 DEFINE_BLOCK_TYPE(GSIndexSetEnumerationBlock, void, NSUInteger, BOOL*);
-- (void)enumerateIndexesInRange: (NSRange)range
-                        options: (NSEnumerationOptions)opts
-		     usingBlock: (GSIndexSetEnumerationBlock)aBlock;
+- (void) enumerateIndexesInRange: (NSRange)range
+                         options: (NSEnumerationOptions)opts
+		      usingBlock: (GSIndexSetEnumerationBlock)aBlock;
 /**
  * Enumerate all indices in the set by applying a block to them.
  */
-- (void)enumerateIndexesUsingBlock: (GSIndexSetEnumerationBlock)aBlock;
+- (void) enumerateIndexesUsingBlock: (GSIndexSetEnumerationBlock)aBlock;
 
-- (void)enumerateIndexesWithOptions: (NSEnumerationOptions)opts
-		         usingBlock: (GSIndexSetEnumerationBlock)aBlock;
+- (void) enumerateIndexesWithOptions: (NSEnumerationOptions)opts
+		          usingBlock: (GSIndexSetEnumerationBlock)aBlock;
 
 #endif
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_7,GS_API_LATEST)
+DEFINE_BLOCK_TYPE(GSIndexSetRangeEnumerationBlock, void, NSRange, BOOL*);
+
+- (void) enumerateRangesInRange: (NSRange)range 
+                        options: (NSEnumerationOptions)opts 
+                     usingBlock: (GSIndexSetRangeEnumerationBlock)aBlock;
+
+- (void) enumerateRangesUsingBlock: (GSIndexSetRangeEnumerationBlock)aBlock;
+
+- (void) enumerateRangesWithOptions: (NSEnumerationOptions)opts 
+                         usingBlock: (GSIndexSetRangeEnumerationBlock)aBlock;
+#endif
+
 /**
  * Returns the first index value in the receiver or NSNotFound if the
  * receiver is empty.
@@ -191,6 +204,7 @@ DEFINE_BLOCK_TYPE(GSIndexSetEnumerationBlock, void, NSUInteger, BOOL*);
 @end
 
 
+GS_EXPORT_CLASS
 @interface	NSMutableIndexSet : NSIndexSet
 
 /**
