@@ -1,7 +1,7 @@
-/* Definition of class NSItemProviderReadingWriting
+/**Definition of class NSItemProviderReadingWriting
    Copyright (C) 2019 Free Software Foundation, Inc.
    
-   By: heron
+   By: Gregory Casamento <greg.casamento@gmail.com>
    Date: Sun Nov 10 04:00:24 EST 2019
 
    This file is part of the GNUstep Library.
@@ -18,8 +18,7 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */
 
 #ifndef _NSItemProviderReadingWriting_h_GNUSTEP_BASE_INCLUDE
