@@ -18,8 +18,7 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */
 
 #ifndef __NSURL_h_GNUSTEP_BASE_INCLUDE
@@ -97,6 +96,18 @@ GS_EXPORT_CLASS
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
 + (instancetype) fileURLWithPath: (NSString*)aPath isDirectory: (BOOL)isDir;
 #endif
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_11, GS_API_LATEST)
++ (instancetype) fileURLWithPath: (NSString *)aPath
+		     isDirectory: (BOOL)isDir
+		   relativeToURL: (NSURL *)baseURL;
+
+/** Create and return a file URL with the supplied path, relative to a base URL.
+ */
++ (instancetype) fileURLWithPath: (NSString *)aPath
+		   relativeToURL: (NSURL *)baseURL;
+#endif
+
 /**
  * Create and return a URL with the supplied string, which should
  * be a string (containing percent escape codes where necessary)
@@ -145,6 +156,33 @@ GS_EXPORT_CLASS
                          isDirectory: (BOOL)isDir;
 #endif
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_11, GS_API_LATEST)
+/**
+ * Initialise as a file URL with the specified path (which must
+ * be a valid path on the local filesystem) relative to the base URL.<br />
+ * Raises NSInvalidArgumentException if aPath is nil.<br />
+ * Converts relative paths to absolute ones.<br />
+ * Appends a trailing slash to the path when necessary if it
+ * specifies a directory.<br />
+ * Calls -initWithScheme:host:path:
+ */
+- (instancetype) initFileURLWithPath: (NSString *)aPath
+		       relativeToURL: (NSURL *)baseURL;
+
+/**
+ * Initialise as a file URL with the specified path (which must
+ * be a valid path on the local filesystem) relative to the base URL.<br />
+ * Raises NSInvalidArgumentException if aPath is nil.<br />
+ * Converts relative paths to absolute ones.<br />
+ * Appends a trailing slash to the path when necessary if it
+ * specifies a directory.<br />
+ * Calls -initWithScheme:host:path:
+ */
+- (instancetype) initFileURLWithPath: (NSString *)aPath
+			 isDirectory: (BOOL)isDir
+		       relativeToURL: (NSURL *)baseURL;
+#endif
+
 /**
  * Initialise by building a URL string from the supplied parameters
  * and calling -initWithString:relativeToURL:<br />
@@ -418,8 +456,8 @@ GS_EXPORT_CLASS
  * directory.<br />
  * See [NSString-stringByAppendingPathComponent:].
  */
-- (NSURL *) URLByAppendingPathComponent:(NSString *)pathComponent
-                            isDirectory:(BOOL)isDirectory;
+- (NSURL *) URLByAppendingPathComponent: (NSString *)pathComponent
+                            isDirectory: (BOOL)isDirectory;
 #endif
 
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
@@ -632,10 +670,10 @@ GS_NSURLQueryItem_IVARS;
 }
 
 // Creating query items.
-+ (instancetype)queryItemWithName:(NSString *)name 
-                            value:(NSString *)value;
-- (instancetype)initWithName:(NSString *)name 
-                       value:(NSString *)value;
++ (instancetype)queryItemWithName: (NSString *)name 
+                            value: (NSString *)value;
+- (instancetype)initWithName: (NSString *)name 
+                       value: (NSString *)value;
 
 // Reading a name and value from a query
 - (NSString *) name;  
@@ -666,14 +704,14 @@ GS_NSURLComponents_IVARS;
 #endif
 }
   // Creating URL components...
-+ (instancetype) componentsWithString:(NSString *)URLString;
-+ (instancetype) componentsWithURL:(NSURL *)url 
-           resolvingAgainstBaseURL:(BOOL)resolve;
++ (instancetype) componentsWithString: (NSString *)URLString;
++ (instancetype) componentsWithURL: (NSURL *)url 
+           resolvingAgainstBaseURL: (BOOL)resolve;
 - (instancetype) init;
-- (instancetype)initWithString:(NSString *)URLString;
+- (instancetype) initWithString: (NSString *)URLString;
 
-- (instancetype)initWithURL:(NSURL *)url 
-    resolvingAgainstBaseURL:(BOOL)resolve;
+- (instancetype) initWithURL: (NSURL *)url 
+     resolvingAgainstBaseURL: (BOOL)resolve;
 
 // Getting the URL
 - (NSString *) string;
