@@ -15,12 +15,11 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */ 
 
 #ifndef __NSUserDefaults_h_OBJECTS_INCLUDE
@@ -46,8 +45,15 @@ extern "C" {
 /* Standard domains */
 
 /**
- *  User defaults domain for process arguments.  Command-line arguments
- *  (attribute-value pairs, as in "-NSFoo bar") are placed into this domain.
+ * User defaults domain for process arguments.  Command-line arguments
+ * (key-value pairs, as in "-NSFoo bar") are placed in this domain.<br />
+ * Where there is a sequence of arguments beginning with '-', only the
+ * last one is used (so "-a" "-b" "-c" "d" will produce a single user default
+ * 'c' with value 'd').<br />
+ * NB. On OSX the argument "-" means a key consisting of an empty string
+ * (so you can't use a '-' as a default value), while in GNUstep a "-" is
+ * a special case which does not mean a default key (so '-' may be used
+ * as a value).<br />
  */
 GS_EXPORT NSString* const NSArgumentDomain;
 
@@ -131,7 +137,7 @@ GS_EXPORT NSString* const NSDecimalDigits;
 /** Key for locale dictionary: array of strings for AM and PM. */
 GS_EXPORT NSString* const NSAMPMDesignation;
 
-#if OS_API_VERSION(GSAPI_MACOSX, GSAPI_LAST)
+#if OS_API_VERSION(GSAPI_MACOSX, GS_API_LATEST)
 
 /**
  *  Array of arrays of NSStrings, first member of each specifying a time,
@@ -185,17 +191,17 @@ GS_EXPORT NSString* const GSLocale;
 #endif
 #endif
 
+#if !NO_GNUSTEP
+/* Path to PEM Encoded Certificate File (NSString). Used by NSURLSession if set. */
+GS_EXPORT NSString *const GSCACertificateFilePath;
+#endif
+
 /* General implementation notes: 
 
    OpenStep spec currently is neither complete nor consistent. Therefore
    we had to make several implementation decisions which may vary in
    other OpenStep implementations.
   
-  - We add a new instance method initWithFile:  as a designated 
-    initialization method because it allows to create user defaults
-    database from a "default user" and also it will work for various 
-    non-posix implementations. 
-
   - We add two new class methods for getting and setting a list of 
     user languages (userLanguages and setUserLanguages: ). They are 
     somehow equivalent to the NS3.x Application's systemLanguages 
@@ -204,15 +210,13 @@ GS_EXPORT NSString* const GSLocale;
   - Definition of argument (command line parameters)
   	(-GSxxxx || --GSxxx) [value]
 	
-    Note:  As far as I know, there is nothing like home directory for 
-    the M$ hell. God help the Win95/WinNT users of NSUserDefaults ;-)
-  
   To Do: 
 	- polish & optimize;
 	- when tested, fix NSBundle (the system languages stuff);
 	- write docs : -(
 	*/
 
+GS_EXPORT_CLASS
 @interface NSUserDefaults : NSObject
 {
 #if	GS_EXPOSE(NSUserDefaults)
@@ -228,8 +232,14 @@ GS_EXPORT NSString* const GSLocale;
   NSRecursiveLock	*_lock;
   NSDistributedLock	*_fileLock;
 #endif
-#if	!GS_NONFRAGILE
-  void			*_unused;
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
 #endif
 }
 
@@ -242,7 +252,7 @@ GS_EXPORT NSString* const GSLocale;
  */
 + (NSUserDefaults*) standardUserDefaults;
 
-#if OS_API_VERSION(GSAPI_MACOSX, GSAPI_LAST)
+#if OS_API_VERSION(GSAPI_MACOSX, GS_API_LATEST)
 /**
  * Resets the shared user defaults object to reflect the current
  * user ID.  Needed by setuid processes which change the user they
@@ -269,7 +279,7 @@ GS_EXPORT NSString* const GSLocale;
 + (void) setUserLanguages: (NSArray*)languages;
 #endif
 
-#if OS_API_VERSION(GSAPI_MACOSX, GSAPI_LAST)
+#if OS_API_VERSION(GSAPI_MACOSX, GS_API_LATEST)
 /**
  * Adds the domain names aName to the search list of the receiver.<br />
  * The domain is added after the application domain.<br />
@@ -308,6 +318,14 @@ GS_EXPORT NSString* const GSLocale;
  */
 - (NSDictionary*) dictionaryForKey: (NSString*)defaultName;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+/**
+ * Looks up a value for a specified default using -objectForKey:
+ * and checks that it is a double.  Returns 0.0 if it is not.
+ */
+- (double) doubleForKey: (NSString*)defaultName;
+#endif
+
 /**
  * Looks up a value for a specified default using -objectForKey:
  * and checks that it is a float.  Returns 0.0 if it is not.
@@ -319,18 +337,12 @@ GS_EXPORT NSString* const GSLocale;
  */
 - (id) init;
 
-/**
- * Initializes defaults for the specified user calling -initWithContentsOfFile:
+/** <init />
+ * Initializes defaults for the specified user.<br />
+ * Returns an object with an empty search list.
  */
 - (id) initWithUser: (NSString*)userName;
 
-/**
- * <init />
- * Initializes defaults for the specified path. Returns an object with
- * an empty search list.
- */
-- (id) initWithContentsOfFile: (NSString*)path;     // This is a new method
-
 /**
  * Looks up a value for a specified default using -objectForKey:
  * and returns its integer value or 0 if it is not representable
@@ -352,7 +364,7 @@ GS_EXPORT NSString* const GSLocale;
  */
 - (void) removeObjectForKey: (NSString*)defaultName;
 
-#if OS_API_VERSION(GSAPI_MACOSX, GSAPI_LAST)
+#if OS_API_VERSION(GSAPI_MACOSX, GS_API_LATEST)
 /**
  * Removes the named domain from the search list of the receiver.<br />
  * Suites may be added using the -addSuiteNamed: method.
@@ -374,6 +386,15 @@ GS_EXPORT NSString* const GSLocale;
  */
 - (void) setBool: (BOOL)value forKey: (NSString*)defaultName;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+/**
+ * Sets a double value for defaultName in the application domain.<br />
+ * Calls -setObject:forKey: to make the change by storing a double
+ * [NSNumber] instance.
+ */
+- (void) setDouble: (double)value forKey: (NSString*)defaultName;
+#endif
+
 /**
  * Sets a float value for defaultName in the application domain.<br />
  * Calls -setObject:forKey: to make the change by storing a float
@@ -451,7 +472,7 @@ GS_EXPORT NSString* const GSLocale;
  * last -synchronize.
  */
 - (void) setPersistentDomain: (NSDictionary*)domain 
-        forName: (NSString*)domainName;
+		     forName: (NSString*)domainName;
 
 /**
  * Ensures that the in-memory and on-disk representations of the defaults
@@ -475,7 +496,7 @@ GS_EXPORT NSString* const GSLocale;
  * exists as either a volatile-domain or a persistent-domain.
  */
 - (void) setVolatileDomain: (NSDictionary*)domain 
-        forName: (NSString*)domainName;
+		   forName: (NSString*)domainName;
 
 /**
  * Returns the volatile domain specified by domainName.
