@@ -18,8 +18,7 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    AutogsdocSource: NSXMLParser.m
    */ 
@@ -36,7 +35,7 @@
 extern "C" {
 #endif
 
-@class NSData, NSDictionary, NSError, NSString, NSURL;
+@class NSData, NSDictionary, NSError, NSInputStream, NSString, NSURL;
 
 /**
  * Domain for errors
@@ -67,6 +66,7 @@ GS_EXPORT NSString* const NSXMLParserErrorDomain;
  *   to handle the parsing process.
  * </p>
  */
+GS_EXPORT_CLASS
 @interface NSXMLParser : NSObject
 {
 #if	GS_EXPOSE(NSXMLParser)
@@ -100,12 +100,17 @@ GS_EXPORT NSString* const NSXMLParserErrorDomain;
 /**
  * Convenience method fetching data from anURL.<br />
  */
-- (id) initWithContentsOfURL: (NSURL*)anURL;
+- (instancetype) initWithContentsOfURL: (NSURL*)anURL;
 
 /** <init />
  * Initialises the parser with the specified xml data.
  */
-- (id) initWithData: (NSData*)data;
+- (instancetype) initWithData: (NSData*)data;
+
+/**
+ * Initialises the parser with the specified input stream.
+ */
+- (instancetype) initWithStream: (NSInputStream*)stream;
 
 /**
  * Parses the supplied data and returns YES on success, NO otherwise.
