@@ -18,8 +18,7 @@
  * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02111 USA. */
+ * Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA. */
 
 #import	<GNUstepBase/GSVersionMacros.h>
 
@@ -46,10 +45,11 @@ extern "C" {
 #endif
 
 /* To easily un-inline functions for debugging */
-#ifndef	INLINE
-#define INLINE inline
+#ifndef GS_STATIC_INLINE
+#define GS_STATIC_INLINE static inline
 #endif
 
+
 #ifdef	GSI_ARRAY_CHECKS
 #define	GSI_ARRAY_CHECK NSCAssert(array->count <= array->cap && array->old <= array->cap, NSInternalInconsistencyException)
 #else
@@ -175,19 +175,28 @@ struct	_GSIArray {
 typedef	struct	_GSIArray	GSIArray_t;
 typedef	struct	_GSIArray	*GSIArray;
 
-static INLINE unsigned
+GS_STATIC_INLINE unsigned
 GSIArrayCapacity(GSIArray array)
 {
   return array->cap;
 }
 
-static INLINE unsigned
+GS_STATIC_INLINE unsigned
 GSIArrayCount(GSIArray array)
 {
   return array->count;
 }
 
-static INLINE void
+GS_STATIC_INLINE NSUInteger
+GSIArraySize(GSIArray array)
+{
+  NSUInteger	size = sizeof(GSIArray);
+
+  size += array->cap * sizeof(GSIArrayItem);
+  return size;
+}
+
+GS_STATIC_INLINE void
 GSIArrayGrow(GSIArray array)
 {
   unsigned int	next;
@@ -207,23 +216,14 @@ GSIArrayGrow(GSIArray array)
 	}
       next = array->cap + array->old;
       size = next*sizeof(GSIArrayItem);
-#if	GS_WITH_GC
-      tmp = NSAllocateCollectable(size, array->zone ? NSScannedOption : 0);
-#else
       tmp = NSZoneMalloc(array->zone, size);
-#endif
       memcpy(tmp, array->ptr, array->count * sizeof(GSIArrayItem));
     }
   else
     {
       next = array->cap + array->old;
       size = next*sizeof(GSIArrayItem);
-#if	GS_WITH_GC
-      tmp = NSReallocateCollectable(array->ptr, size,
-	array->zone ? NSScannedOption : 0);
-#else
       tmp = NSZoneRealloc(array->zone, array->ptr, size);
-#endif
     }
 
   if (tmp == 0)
@@ -236,7 +236,7 @@ GSIArrayGrow(GSIArray array)
   array->cap = next;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayGrowTo(GSIArray array, unsigned next)
 {
   unsigned int	size;
@@ -253,21 +253,12 @@ GSIArrayGrowTo(GSIArray array, unsigned next)
       /*
        * Statically initialised buffer ... copy into new heap buffer.
        */
-#if	GS_WITH_GC
-      tmp = NSAllocateCollectable(size, array->zone ? NSScannedOption : 0);
-#else
       tmp = NSZoneMalloc(array->zone, size);
-#endif
       memcpy(tmp, array->ptr, array->count * sizeof(GSIArrayItem));
     }
   else
     {
-#if	GS_WITH_GC
-      tmp = NSReallocateCollectable(array->ptr, size,
-	array->zone ? NSScannedOption : 0);
-#else
       tmp = NSZoneRealloc(array->zone, array->ptr, size);
-#endif
     }
 
   if (tmp == 0)
@@ -280,7 +271,7 @@ GSIArrayGrowTo(GSIArray array, unsigned next)
   array->cap = next;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayInsertItem(GSIArray array, GSIArrayItem item, unsigned index)
 {
   unsigned int	i;
@@ -299,7 +290,7 @@ GSIArrayInsertItem(GSIArray array, GSIArrayItem item, unsigned index)
   GSI_ARRAY_CHECK;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayInsertItemNoRetain(GSIArray array, GSIArrayItem item, unsigned index)
 {
   unsigned int	i;
@@ -317,7 +308,7 @@ GSIArrayInsertItemNoRetain(GSIArray array, GSIArrayItem item, unsigned index)
   GSI_ARRAY_CHECK;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayAddItem(GSIArray array, GSIArrayItem item)
 {
   GSI_ARRAY_CHECK;
@@ -330,7 +321,7 @@ GSIArrayAddItem(GSIArray array, GSIArrayItem item)
   GSI_ARRAY_CHECK;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayAddItemNoRetain(GSIArray array, GSIArrayItem item)
 {
   GSI_ARRAY_CHECK;
@@ -349,7 +340,7 @@ GSIArrayAddItemNoRetain(GSIArray array, GSIArrayItem item)
  *      added is 'less than' the item in the array, NSOrderedDescending
  *      if it is greater, and NSOrderedSame if it is equal.
  */
-static INLINE unsigned
+GS_STATIC_INLINE unsigned
 GSIArraySearch(GSIArray array, GSIArrayItem item, 
   NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
@@ -382,7 +373,7 @@ GSIArraySearch(GSIArray array, GSIArrayItem item,
   return index;
 }
 
-static INLINE unsigned
+GS_STATIC_INLINE unsigned
 GSIArrayInsertionPosition(GSIArray array, GSIArrayItem item, 
   NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
@@ -405,7 +396,7 @@ GSIArrayInsertionPosition(GSIArray array, GSIArrayItem item,
 }
 
 #ifdef	GSI_ARRAY_CHECKS
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayCheckSort(GSIArray array, 
   NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
@@ -421,7 +412,7 @@ GSIArrayCheckSort(GSIArray array,
 }
 #endif
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayInsertSorted(GSIArray array, GSIArrayItem item, 
   NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
@@ -437,7 +428,7 @@ GSIArrayInsertSorted(GSIArray array, GSIArrayItem item,
 #endif
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayInsertSortedNoRetain(GSIArray array, GSIArrayItem item,
   NSComparisonResult (*sorter)(GSIArrayItem, GSIArrayItem))
 {
@@ -453,80 +444,84 @@ GSIArrayInsertSortedNoRetain(GSIArray array, GSIArrayItem item,
 #endif
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayRemoveItemAtIndex(GSIArray array, unsigned index)
 {
-  GSIArrayItem	tmp;
-#ifdef	GSI_ARRAY_CHECKS
+#if	defined(GSI_ARRAY_NO_RELEASE)
+# ifdef	GSI_ARRAY_CHECKS
   NSCAssert(index < array->count, NSInvalidArgumentException);
-#endif
+# endif
+  while (++index < array->count)
+    array->ptr[index-1] = array->ptr[index];
+  array->count--;
+#else
+  GSIArrayItem	tmp;
+# ifdef	GSI_ARRAY_CHECKS
+  NSCAssert(index < array->count, NSInvalidArgumentException);
+# endif
   tmp = array->ptr[index];
   while (++index < array->count)
     array->ptr[index-1] = array->ptr[index];
   array->count--;
-#if	!defined(GS_NO_RELEASE)
-#if	GS_WITH_GC
-  array->ptr[array->count] = (GSIArrayItem)(NSUInteger)0;
-#endif
   GSI_ARRAY_RELEASE(array, tmp);
 #endif
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayRemoveLastItem(GSIArray array)
 {
 #ifdef	GSI_ARRAY_CHECKS
   NSCAssert(array->count, NSInvalidArgumentException);
 #endif
   array->count--;
-#if	!defined(GS_NO_RELEASE)
+#if	!defined(GSI_ARRAY_NO_RELEASE)
   GSI_ARRAY_RELEASE(array, array->ptr[array->count]);
-#if	GS_WITH_GC
-  array->ptr[array->count] = (GSIArrayItem)(NSUInteger)0;
-#endif
 #endif
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayRemoveItemAtIndexNoRelease(GSIArray array, unsigned index)
 {
-  GSIArrayItem	tmp;
 #ifdef	GSI_ARRAY_CHECKS
   NSCAssert(index < array->count, NSInvalidArgumentException);
 #endif
-  tmp = array->ptr[index];
   while (++index < array->count)
     array->ptr[index-1] = array->ptr[index];
   array->count--;
-#if	GS_WITH_GC && !defined(GS_NO_RELEASE)
-  array->ptr[array->count] = (GSIArrayItem)(NSUInteger)0;
-#endif
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArraySetItemAtIndex(GSIArray array, GSIArrayItem item, unsigned index)
 {
-  GSIArrayItem	tmp;
-#ifdef	GSI_ARRAY_CHECKS
+#if	defined(GSI_ARRAY_NO_RELEASE)
+# ifdef	GSI_ARRAY_CHECKS
   NSCAssert(index < array->count, NSInvalidArgumentException);
-#endif
+# endif
+  GSI_ARRAY_RETAIN(array, item);
+  array->ptr[index] = item;
+#else
+  GSIArrayItem	tmp;
+# ifdef	GSI_ARRAY_CHECKS
+  NSCAssert(index < array->count, NSInvalidArgumentException);
+# endif
   tmp = array->ptr[index];
   GSI_ARRAY_RETAIN(array, item);
   array->ptr[index] = item;
   GSI_ARRAY_RELEASE(array, tmp);
+#endif
 }
 
 /*
  * For direct access ... unsafe if you change the array in any way while
  * examining the contents of this buffer.
  */
-static INLINE GSIArrayItem *
+GS_STATIC_INLINE GSIArrayItem *
 GSIArrayItems(GSIArray array)
 {
   return array->ptr;
 }
 
-static INLINE GSIArrayItem
+GS_STATIC_INLINE GSIArrayItem
 GSIArrayItemAtIndex(GSIArray array, unsigned index)
 {
 #ifdef	GSI_ARRAY_CHECKS
@@ -535,7 +530,7 @@ GSIArrayItemAtIndex(GSIArray array, unsigned index)
   return array->ptr[index];
 }
 
-static INLINE GSIArrayItem
+GS_STATIC_INLINE GSIArrayItem
 GSIArrayLastItem(GSIArray array)
 {
 #ifdef	GSI_ARRAY_CHECKS
@@ -544,12 +539,11 @@ GSIArrayLastItem(GSIArray array)
   return array->ptr[array->count-1];
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayClear(GSIArray array)
 {
   if (array->ptr)
     {
-#if	!GS_WITH_GC
       /*
        * Only free memory if it was dynamically initialised (old > 0)
        */
@@ -557,13 +551,12 @@ GSIArrayClear(GSIArray array)
 	{
 	  NSZoneFree(array->zone, (void*)array->ptr);
 	}
-#endif
       array->ptr = 0;
       array->cap = 0;
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayRemoveItemsFromIndex(GSIArray array, unsigned index)
 {
   if (index < array->count)
@@ -572,38 +565,32 @@ GSIArrayRemoveItemsFromIndex(GSIArray array, unsigned index)
       while (array->count-- > index)
 	{
 	  GSI_ARRAY_RELEASE(array, array->ptr[array->count]);
-#if	GS_WITH_GC
-  	  array->ptr[array->count] = (GSIArrayItem)(NSUInteger)0;
-#endif
 	}
 #endif
       array->count = index;
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayRemoveAllItems(GSIArray array)
 {
 #ifndef	GSI_ARRAY_NO_RELEASE
   while (array->count--)
     {
       GSI_ARRAY_RELEASE(array, array->ptr[array->count]);
-#if	GS_WITH_GC
-      array->ptr[array->count] = (GSIArrayItem)(NSUInteger)0;
-#endif
     }
 #endif
   array->count = 0;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIArrayEmpty(GSIArray array)
 {
   GSIArrayRemoveAllItems(array);
   GSIArrayClear(array);
 }
 
-static INLINE GSIArray
+GS_STATIC_INLINE GSIArray
 GSIArrayInitWithZoneAndCapacity(GSIArray array, NSZone *zone, size_t capacity)
 {
   unsigned int	size;
@@ -615,16 +602,11 @@ GSIArrayInitWithZoneAndCapacity(GSIArray array, NSZone *zone, size_t capacity)
   array->cap = capacity;
   array->old = capacity/2;
   size = capacity*sizeof(GSIArrayItem);
-#if	GS_WITH_GC
-  array->ptr = (GSIArrayItem*)NSAllocateCollectable(size,
-    array->zone ? NSScannedOption : 0);
-#else
   array->ptr = (GSIArrayItem*)NSZoneMalloc(zone, size);
-#endif
   return array;
 }
 
-static INLINE GSIArray
+GS_STATIC_INLINE GSIArray
 GSIArrayInitWithZoneAndStaticCapacity(GSIArray array, NSZone *zone,
     size_t capacity, GSIArrayItem *buffer)
 {
@@ -636,18 +618,13 @@ GSIArrayInitWithZoneAndStaticCapacity(GSIArray array, NSZone *zone,
   return array;
 }
 
-static INLINE GSIArray
+GS_STATIC_INLINE GSIArray
 GSIArrayCopyWithZone(GSIArray array, NSZone *zone)
 {
   unsigned int i;
   GSIArray new;
 
-#if	GS_WITH_GC
-  new = NSAllocateCollectable(sizeof(GSIArray_t), NSScannedOption);
-#else
   new = NSZoneMalloc(zone, sizeof(GSIArray_t));
-#endif
-
   GSIArrayInitWithZoneAndCapacity(new, zone, array->count);
   for (i = 0; i < array->count; i++)
     {
